"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _pluginSyntaxFlow = _interopRequireDefault(require("@babel/plugin-syntax-flow"));

var t = _interopRequireWildcard(require("@babel/types"));

var _lodash = _interopRequireDefault(require("lodash.snakecase"));

var _lodash2 = _interopRequireDefault(require("lodash.camelcase"));

var _babelAddFlowComments = _interopRequireDefault(require("babel-add-flow-comments"));

var _s2sUtils = require("s2s-utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import blog from 'babel-log'
const constantCase = string => (0, _lodash.default)(string).toUpperCase();

const builders = {
  actionCreater: (0, _s2sUtils.template)(`export function NAME(PARAMS): TYPE {
    return VALUE;
  }`)
};

function createActionCreater(name, properties, params) {
  const typeIdentifier = t.identifier('type');
  return builders.actionCreater({
    NAME: t.identifier((0, _lodash2.default)(name)),
    TYPE: t.identifier(name),
    VALUE: t.objectExpression([t.objectProperty(typeIdentifier, t.identifier(constantCase(name))), ...properties]),
    PARAMS: params
  });
}

var _default = () => {
  return {
    inherits: _pluginSyntaxFlow.default,
    name: 's2s-action-creater',
    visitor: {
      Program: {
        exit(programPath, state) {
          const {
            file
          } = state;

          const basename = _path.default.parse(file.opts.generatorOpts.filename).name;

          const imports = [];
          const typeNames = [];
          const actions = [];
          const funcs = [];
          programPath.traverse({
            ImportDeclaration(path) {
              imports.push(path.node);
            },

            VariableDeclarator(path) {
              const name = path.get('id').node.name;

              if (name !== 'Actions') {
                actions.push(name);
              }
            },

            TypeAlias(path) {
              const name = path.get('id').node.name;

              if (name === 'Action') {
                return;
              }

              typeNames.push(name);
              const params = [];
              const properties = [];

              for (const _ref of path.get('right').node.properties) {
                const {
                  key,
                  value
                } = _ref;

                if (key.name !== 'type') {
                  const p = t.identifier(key.name);
                  properties.push(t.objectProperty(p, p, false, true));
                  p.typeAnnotation = t.typeAnnotation(value);
                  params.push(p);
                }
              }

              funcs.push(createActionCreater(name, properties, params));
            }

          });
          const constImport = (0, _s2sUtils.createImportDeclaration)(actions, `./${basename}`);
          const typeImport = (0, _s2sUtils.createImportDeclaration)(typeNames, `./${basename}`);
          typeImport.importKind = 'type';
          programPath.node.body = [...imports, constImport, typeImport, t.noop(), ...funcs];
          (0, _babelAddFlowComments.default)(programPath);
        }

      }
    }
  };
};

exports.default = _default;